@@ -914,6 +914,186 @@ do_connection_event_test (void)
 	soup_test_session_abort_unref (session);
 }
 
+typedef struct {
+        GMainLoop *loop;
+        GIOStream *stream;
+        GError *error;
+        const char *events;
+} ConnectTestData;
+
+static void
+connect_progress (SoupSession *session, GSocketClientEvent event, GIOStream *connection, ConnectTestData *data)
+{
+        soup_test_assert (*data->events == event_abbrevs[event],
+                          "Unexpected event: %s (expected %s)",
+                          event_names[event],
+                          event_name_from_abbrev (*data->events));
+        data->events = data->events + 1;
+}
+
+static void
+connect_finished (SoupSession *session, GAsyncResult *result, ConnectTestData *data)
+{
+        data->stream = soup_session_connect_finish (session, result, &data->error);
+        g_main_loop_quit (data->loop);
+}
+
+static void
+do_one_connection_connect_test (SoupSession *session, SoupURI *uri, const char *response, const char *events)
+{
+        ConnectTestData data = { NULL, NULL, NULL, events };
+        static const char *request = "GET / HTTP/1.1\r\nHost: localhost\r\n\r\n";
+        gsize bytes = 0;
+        char buffer[128];
+
+        data.loop = g_main_loop_new (NULL, FALSE);
+        soup_session_connect_async (session, uri, NULL,
+                                    (SoupSessionConnectProgressCallback)connect_progress,
+                                    (GAsyncReadyCallback)connect_finished,
+                                    &data);
+        g_main_loop_run (data.loop);
+
+        g_assert (G_IS_IO_STREAM (data.stream));
+        g_assert_no_error (data.error);
+        g_assert (g_output_stream_write_all (g_io_stream_get_output_stream (data.stream),
+                                             request, strlen (request), &bytes, NULL, NULL));
+        g_assert (g_input_stream_read_all (g_io_stream_get_input_stream (data.stream),
+                                           buffer, sizeof (buffer), &bytes, NULL, NULL));
+        buffer[strlen (response)] = '\0';
+        g_assert_cmpstr (buffer, ==, response);
+
+        while (*data.events) {
+                soup_test_assert (!*data.events,
+                                  "Expected %s",
+                                  event_name_from_abbrev (*data.events));
+                data.events++;
+        }
+
+        g_object_unref (data.stream);
+        g_main_loop_unref (data.loop);
+}
+
+static void
+do_one_connection_connect_fail_test (SoupSession *session, SoupURI *uri, GQuark domain, gint code, const char *events)
+{
+        ConnectTestData data = { NULL, NULL, NULL, events };
+
+        data.loop = g_main_loop_new (NULL, FALSE);
+        soup_session_connect_async (session, uri, NULL,
+                                    (SoupSessionConnectProgressCallback)connect_progress,
+                                    (GAsyncReadyCallback)connect_finished,
+                                    &data);
+        g_main_loop_run (data.loop);
+
+        g_assert (!data.stream);
+        g_assert_error (data.error, domain, code);
+
+        while (*data.events) {
+                soup_test_assert (!*data.events,
+                                  "Expected %s",
+                                  event_name_from_abbrev (*data.events));
+                data.events++;
+        }
+}
+
+static void
+do_connection_connect_test (void)
+{
+        SoupSession *session;
+        SoupURI *http_uri;
+        SoupURI *https_uri = NULL;
+        SoupURI *ws_uri;
+        SoupURI *wss_uri = NULL;
+        SoupURI *file_uri;
+        SoupURI *wrong_http_uri;
+        SoupURI *proxy_uri;
+
+        SOUP_TEST_SKIP_IF_NO_APACHE;
+
+        session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
+                                         SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
+                                         NULL);
+
+        debug_printf (1, "    http\n");
+        http_uri = soup_uri_new (HTTP_SERVER);
+        do_one_connection_connect_test (session, http_uri,
+                                        "HTTP/1.1 200 OK", "rRcCx");
+
+        if (tls_available) {
+                debug_printf (1, "    https\n");
+                https_uri = soup_uri_new (HTTPS_SERVER);
+                do_one_connection_connect_test (session, https_uri,
+                                                "HTTP/1.1 200 OK", "rRcCtTx");
+        } else
+                debug_printf (1, "    https -- SKIPPING\n");
+
+        debug_printf (1, "    ws\n");
+        ws_uri = soup_uri_new (HTTP_SERVER);
+        ws_uri->scheme = SOUP_URI_SCHEME_WS;
+        do_one_connection_connect_test (session, ws_uri,
+                                        "HTTP/1.1 200 OK", "rRcCx");
+
+        if (tls_available) {
+                debug_printf (1, "    wss\n");
+                wss_uri = soup_uri_new (HTTPS_SERVER);
+                do_one_connection_connect_test (session, wss_uri,
+                                                "HTTP/1.1 200 OK", "rRcCtTx");
+        } else
+                debug_printf (1, "    wss -- SKIPPING\n");
+
+        debug_printf (1, "    file\n");
+        file_uri = soup_uri_new ("file:///foo/bar");
+        do_one_connection_connect_fail_test (session, file_uri,
+                                             G_RESOLVER_ERROR, G_RESOLVER_ERROR_NOT_FOUND,
+                                             "r");
+
+        debug_printf (1, "    wrong http (invalid port)\n");
+        wrong_http_uri = soup_uri_new (HTTP_SERVER);
+        wrong_http_uri->port = 1234;
+        do_one_connection_connect_fail_test (session, wrong_http_uri,
+                                             G_IO_ERROR, G_IO_ERROR_CONNECTION_REFUSED,
+                                             "rRcr"); /* FIXME: why r again? GLib bug? */
+
+        proxy_uri = soup_uri_new (HTTP_PROXY);
+        g_object_set (G_OBJECT (session),
+                      SOUP_SESSION_PROXY_URI, proxy_uri,
+                      NULL);
+
+        debug_printf (1, "    http with proxy\n");
+        do_one_connection_connect_test (session, http_uri,
+                                        "HTTP/1.1 403 Forbidden", "rRcCx");
+
+        if (tls_available) {
+                debug_printf (1, "    https with proxy\n");
+                do_one_connection_connect_test (session, https_uri,
+                                                "HTTP/1.1 200 OK", "rRcCpPtTx");
+        } else
+                debug_printf (1, "    https with proxy -- SKIPPING\n");
+
+        debug_printf (1, "    ws with proxy\n");
+        do_one_connection_connect_test (session, ws_uri,
+                                        "HTTP/1.1 403 Forbidden", "rRcCx");
+
+        if (tls_available) {
+                debug_printf (1, "    wss with proxy\n");
+                do_one_connection_connect_test (session, wss_uri,
+                                                "HTTP/1.1 200 OK", "rRcCpPtTx");
+        } else
+                debug_printf (1, "    wss with proxy -- SKIPPING\n");
+
+        soup_uri_free (http_uri);
+        if (https_uri)
+                soup_uri_free (https_uri);
+        soup_uri_free (ws_uri);
+        if (wss_uri)
+                soup_uri_free (wss_uri);
+        soup_uri_free (file_uri);
+        soup_uri_free (wrong_http_uri);
+        soup_uri_free (proxy_uri);
+
+        soup_test_session_abort_unref (session);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -933,6 +1113,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/connection/non-idempotent", do_non_idempotent_connection_test);
 	g_test_add_func ("/connection/state", do_connection_state_test);
 	g_test_add_func ("/connection/event", do_connection_event_test);
+	g_test_add_func ("/connection/connect", do_connection_connect_test);
 
 	ret = g_test_run ();
 
